#ifndef _B3SOIFDINIT_H
#define _B3SOIFDINIT_H

extern IFparm B3SOIFDpTable[];
extern IFparm B3SOIFDmPTable[];
extern char *B3SOIFDnames[];
extern int B3SOIFDpTSize;
extern int B3SOIFDmPTSize;
extern int B3SOIFDnSize;
extern int B3SOIFDiSize;
extern int B3SOIFDmSize;

#endif
