/*============================================================================
FILE    MIFdestroy.c

MEMBER OF process XSPICE

Public Domain

Georgia Tech Research Corporation
Atlanta, Georgia 30332
PROJECT A-8503

AUTHORS

    9/12/91  Bill Kuhn

MODIFICATIONS

    <date> <person name> <nature of modifications>

SUMMARY

    This file contains a function that deletes all models of a particular
    device (code model) type from the circuit description structures.

INTERFACES

    MIFdestroy()

REFERENCED FILES

    None.

NON-STANDARD FEATURES

    None.

============================================================================*/

/* #include "prefix.h" */
#include "ngspice/ngspice.h"
#include <stdio.h>

#include "ngspice/mifproto.h"

/* #include "suffix.h"  */



/*
MIFdestroy
*/

void MIFdestroy(void)
{
}
