/**********
Copyright 2010 Regents of the University of California.  All rights reserved.
Author: 2005 Hui Wan (based on Samuel Fung's b3soiitf.h)
Authors: 2009- Wenwei Yang, Chung-Hsun Lin, Ali Niknejad, Chenming Hu.
Authors: 2009- Tanvir Morshed, Ali Niknejad, Chenming Hu.
Authors: 2010- Tanvir Morshed, Ali Niknejad, Chenming Hu.
File: b4soiitf.h
**********/

#ifndef DEV_B4SOI
#define DEV_B4SOI

#include "b4soiext.h"

SPICEdev *get_b4soi_info (void);

#endif

