/**********
License              : 3-clause BSD
Spice3 Implementation: 2019-2020 Dietmar Warning, Markus Müller, Mario Krattenmacher
Model Author         : (Copyright 1993-2024) Michael Schroter
**********/


#ifndef _HICUMINIT_H
#define _HICUMINIT_H

extern IFparm HICUMpTable[ ];
extern IFparm HICUMmPTable[ ];
extern char *HICUMnames[ ];
extern int HICUMpTSize;
extern int HICUMmPTSize;
extern int HICUMnSize;
extern int HICUMiSize;
extern int HICUMmSize;

#endif
