extern  SPICEdev  cm_aswitch_info;
extern  SPICEdev  cm_capacitor_info;
extern  SPICEdev  cm_cmeter_info;
extern  SPICEdev  cm_core_info;
extern  SPICEdev  cm_inductor_info;
extern  SPICEdev  cm_lcouple_info;
extern  SPICEdev  cm_lmeter_info;
extern  SPICEdev  cm_potentiometer_info;
extern  SPICEdev  cm_zener_info;
extern  SPICEdev  cm_memristor_info;
extern  SPICEdev  cm_sidiode_info;
extern  SPICEdev  cm_pswitch_info;
extern  SPICEdev  cm_seegen_info;
