#ifndef _NUMDINIT_H
#define _NUMDINIT_H

extern IFparm NUMDpTable[ ];
extern IFparm NUMDmPTable[ ];
extern char *NUMDnames[ ];
extern int NUMDpTSize;
extern int NUMDmPTSize;
extern int NUMDnSize;
extern int NUMDiSize;
extern int NUMDmSize;

#endif
