#ifndef _INDINIT_H
#define _INDINIT_H

extern IFparm INDpTable[ ];
extern IFparm INDmPTable[ ];
extern char *INDnames[ ];
extern int INDpTSize;
extern int INDmPTSize;

extern int INDnSize;
extern int INDiSize;
extern int INDmSize;

extern IFparm MUTpTable[ ];
extern int MUTpTSize;
extern int MUTiSize;
extern int MUTmSize;

#endif
