extern  SPICEdev  cm_adc_bridge_info;
extern  SPICEdev  cm_dac_bridge_info;
extern  SPICEdev  cm_bidi_bridge_info;
extern  SPICEdev  cm_d_and_info;
extern  SPICEdev  cm_d_buffer_info;
extern  SPICEdev  cm_d_dff_info;
extern  SPICEdev  cm_d_dlatch_info;
extern  SPICEdev  cm_d_fdiv_info;
extern  SPICEdev  cm_d_genlut_info;
extern  SPICEdev  cm_d_inverter_info;
extern  SPICEdev  cm_d_jkff_info;
extern  SPICEdev  cm_d_lut_info;
extern  SPICEdev  cm_d_nand_info;
extern  SPICEdev  cm_d_nor_info;
extern  SPICEdev  cm_d_open_c_info;
extern  SPICEdev  cm_d_open_e_info;
extern  SPICEdev  cm_d_or_info;
extern  SPICEdev  cm_d_osc_info;
extern  SPICEdev  cm_d_process_info;
extern  SPICEdev  cm_d_pulldown_info;
extern  SPICEdev  cm_d_pullup_info;
extern  SPICEdev  cm_d_pwm_info;
extern  SPICEdev  cm_d_ram_info;
extern  SPICEdev  cm_d_source_info;
extern  SPICEdev  cm_d_srff_info;
extern  SPICEdev  cm_d_srlatch_info;
extern  SPICEdev  cm_d_state_info;
extern  SPICEdev  cm_d_tff_info;
extern  SPICEdev  cm_d_tristate_info;
extern  SPICEdev  cm_d_xnor_info;
extern  SPICEdev  cm_d_xor_info;
extern  SPICEdev  ucm_d_cosim_info;
