/**********
Copyright 1990 Regents of the University of California.  All rights
reserved.
Author: 1990 Jaijeet S. Roychowdhury
**********/
#ifndef DEV_LTRA
#define DEV_LTRA

SPICEdev *get_ltra_info(void);

#endif
