#ifndef _NBJTINIT_H
#define _NBJTINIT_H

extern IFparm NBJTpTable[ ];
extern IFparm NBJTmPTable[ ];
extern char *NBJTnames[ ];
extern int NBJTpTSize;
extern int NBJTmPTSize;
extern int NBJTnSize;
extern int NBJTiSize;
extern int NBJTmSize;

#endif
