/**********
 (C) Paolo Nenzi 2001
 **********/ 

/*
 * Bernoulli function
 */
 
#ifndef ngspice_BERNOULL_H
#define ngspice_BERNOULL_H

extern void bernoulli (double, double *, double *, double *, 
                       double *, bool);

#endif
