extern  SPICEdev  cm_climit_info;
extern  SPICEdev  cm_divide_info;
extern  SPICEdev  cm_d_dt_info;
extern  SPICEdev  cm_gain_info;
extern  SPICEdev  cm_hyst_info;
extern  SPICEdev  cm_ilimit_info;
extern  SPICEdev  cm_int_info;
extern  SPICEdev  cm_limit_info;
extern  SPICEdev  cm_mult_info;
extern  SPICEdev  cm_multi_input_pwl_info;
extern  SPICEdev  cm_oneshot_info;
extern  SPICEdev  cm_pwl_info;
extern  SPICEdev  cm_sine_info;
extern  SPICEdev  cm_slew_info;
extern  SPICEdev  cm_square_info;
extern  SPICEdev  cm_summer_info;
extern  SPICEdev  cm_xfer_info;
extern  SPICEdev  cm_s_xfer_info;
extern  SPICEdev  cm_triangle_info;
extern  SPICEdev  cm_filesource_info;
extern  SPICEdev  cm_delay_info;
extern  SPICEdev  cm_pwlts_info;
extern  SPICEdev  cm_astate_info;
