#ifndef _MOS1INIT_H
#define _MOS1INIT_H

extern IFparm MOS1pTable[ ];
extern IFparm MOS1mPTable[ ];
extern char *MOS1names[ ];
extern int MOS1pTSize;
extern int MOS1mPTSize;
extern int MOS1nSize;
extern int MOS1iSize;
extern int MOS1mSize;

#endif
