#ifndef _MOS2INIT_H
#define _MOS2INIT_H

extern IFparm MOS2pTable[ ];
extern IFparm MOS2mPTable[ ];
extern char *MOS2names[ ];
extern int MOS2pTSize;
extern int MOS2mPTSize;
extern int MOS2nSize;
extern int MOS2iSize;
extern int MOS2mSize;

#endif
