/**********
Based on jfetitf.h
Copyright 1990 Regents of the University of California.  All rights reserved.

Modified to add PS model and new parameter definitions ( Anthony E. Parker )
   Copyright 1994  Macquarie University, Sydney Australia.
   pz and disto not supported
**********/
#ifndef DEV_JFET2
#define DEV_JFET2

SPICEdev *get_jfet2_info(void);

#endif
