/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
**********/
#ifndef DEV_IND
#define DEV_IND

SPICEdev *get_ind_info(void);
SPICEdev *get_mut_info(void);

#endif
