#ifndef ngspice_CONF_H
#define ngspice_CONF_H

extern char Spice_Version[];
extern char Spice_Notice[];
extern char Spice_Build_Date[];
extern char Spice_Manual[];
extern char *Spice_Exec_Dir;
extern char *Spice_Lib_Dir;
extern char *Def_Editor;
extern int AsciiRawFile;

extern char *Bug_Addr;
extern char *Spice_Host;
extern char *Spiced_Log;

#endif
