#ifndef _LTRAINIT_H
#define _LTRAINIT_H

extern IFparm LTRApTable[ ];
extern IFparm LTRAmPTable[ ];
extern char *LTRAnames[ ];
extern int LTRApTSize;
extern int LTRAmPTSize;
extern int LTRAnSize;
extern int LTRAiSize;
extern int LTRAmSize;

#endif
