/**********
Copyright 1999 Regents of the University of California.  All rights reserved.
Author: 1998 Samuel Fung
File: b3soidditf.h
**********/

#ifndef DEV_B3SOIDD
#define DEV_B3SOIDD

#include "b3soiddext.h"

SPICEdev *get_b3soidd_info (void);

#endif
