/**********
License              : 3-clause BSD
Spice3 Implementation: 2019-2020 Dietmar Warning, Markus Müller, Mario Krattenmacher
Model Author         : (Copyright 1993-2024) Michael Schroter
**********/
#ifndef DEV_HICUM
#define DEV_HICUM

extern SPICEdev *get_hicum_info(void);

#endif
