#ifndef _BSIM1INIT_H
#define _BSIM1INIT_H

extern IFparm B1pTable[ ];
extern IFparm B1mPTable[ ];
extern char *B1names[ ];
extern int B1pTSize;
extern int B1mPTSize;
extern int B1nSize;
extern int B1iSize;
extern int B1mSize;

#endif
