#ifndef _MOS9INIT_H
#define _MOS9INIT_H

extern IFparm MOS9pTable[ ];
extern IFparm MOS9mPTable[ ];
extern char *MOS9names[ ];
extern int MOS9pTSize;
extern int MOS9mPTSize;
extern int MOS9nSize;
extern int MOS9iSize;
extern int MOS9mSize;

#endif
