/*************
 * Header file for com_alias.c
 * 1999 E. Rouat
 ************/

#ifndef ngspice_COM_ALIAS_H
#define ngspice_COM_ALIAS_H

#include "ngspice/wordlist.h"

void com_alias(wordlist *wl);
void com_unalias(wordlist *wl);

#endif
