/************************************************/
/******  Structures, etc. for pwl model.   ******/
/******  10/10/90                    JPM   ******/
/************************************************/



/****  Error Messages ****/

char *oneshot_allocation_error = "\n**** Error ****\nONESHOT: Error allocating oneshot block storage \n";
char *oneshot_array_error = "\n**** Error ****\nONESHOT: Size of control array different than pulse-width array \n";
char *oneshot_pw_clamp = "\n**** Warning ****\nONESHOT: Extrapolated Pulse-Width Limited to zero \n";

                                    
#define T1 1
#define T2 2
#define T3 3
#define T4 4
#define SET 5
#define STATE 6
#define CLOCK 7
#define OUTPUT_OLD 8
#define LOCKED 9

/***** Define Error Messages **************************************/






/*****  Structure Definitions *************************************/
  




                                   
/***** Function Definitions ***************************************/




