#ifndef _MOS3INIT_H
#define _MOS3INIT_H

extern IFparm MOS3pTable[ ];
extern IFparm MOS3mPTable[ ];
extern char *MOS3names[ ];
extern int MOS3pTSize;
extern int MOS3mPTSize;
extern int MOS3nSize;
extern int MOS3iSize;
extern int MOS3mSize;

#endif
