#ifndef _URCINIT_H
#define _URCINIT_H

extern IFparm URCpTable[ ];
extern IFparm URCmPTable[ ];
extern char *URCnames[ ];
extern int URCpTSize;
extern int URCmPTSize;
extern int URCnSize;
extern int URCiSize;
extern int URCmSize;

#endif
